@@ -58,6 +58,13 @@ static const char *col_red    = "\e[01;31m";
 static const char *col_redinv = "\e[01;07;31m";
 static const char *col_end    = "\e[0m";
 
+/* Some gcc 3.x, 4.[01].X crash with internal compiler error. */
+#if __GNUC__ <= 3 || (__GNUC__ == 4 && __GNUC_MINOR__ <= 1)
+# define  BROKEN_GCC_FORMAT_ATTRIBUTE
+#else
+# define  BROKEN_GCC_FORMAT_ATTRIBUTE __attribute__((format(printf, 6, 0)))
+#endif
+
 /**
  * struct ntfs_logging - Control info for the logging system
  * @levels:	Bitfield of logging levels
@@ -67,7 +74,7 @@ static const char *col_end    = "\e[0m";
 struct ntfs_logging {
 	u32 levels;
 	u32 flags;
-	ntfs_log_handler *handler;
+	ntfs_log_handler *handler BROKEN_GCC_FORMAT_ATTRIBUTE;
 };
 
 /**
@@ -355,14 +362,14 @@ int ntfs_log_handler_syslog(const char *function  __attribute__((unused)),
 			    void *data __attribute__((unused)), 
 			    const char *format, va_list args)
 {
-	char log[LOG_LINE_LEN];
+	char logbuf[LOG_LINE_LEN];
 	int ret, olderr = errno;
 
 #ifndef DEBUG
 	if ((level & NTFS_LOG_LEVEL_PERROR) && errno == ENOSPC)
 		return 1;
 #endif	
-	ret = vsnprintf(log, LOG_LINE_LEN, format, args);
+	ret = vsnprintf(logbuf, LOG_LINE_LEN, format, args);
 	if (ret < 0) {
 		vsyslog(LOG_NOTICE, format, args);
 		ret = 1;
@@ -370,12 +377,12 @@ int ntfs_log_handler_syslog(const char *function  __attribute__((unused)),
 	}
 	
 	if ((LOG_LINE_LEN > ret + 3) && (level & NTFS_LOG_LEVEL_PERROR)) {
-		strncat(log, ": ", LOG_LINE_LEN - ret - 1);
-		strncat(log, strerror(olderr), LOG_LINE_LEN - (ret + 3));
-		ret = strlen(log);
+		strncat(logbuf, ": ", LOG_LINE_LEN - ret - 1);
+		strncat(logbuf, strerror(olderr), LOG_LINE_LEN - (ret + 3));
+		ret = strlen(logbuf);
 	}
 	
-	syslog(LOG_NOTICE, "%s", log);
+	syslog(LOG_NOTICE, "%s", logbuf);
 out:
 	errno = olderr;
 	return ret;
