@@ -544,9 +544,8 @@ runlist_element *ntfs_runlists_merge(runlist_element *drl,
 
 	/* Can't have an entirely unmapped source runlist. */
 	if (!srl[si].length) {
-		ntfs_log_debug("Eeek! ntfs_runlists_merge() received entirely "
-				"unmapped source runlist.\n");
 		errno = EINVAL;
+		ntfs_log_perror("%s: unmapped source runlist", __FUNCTION__);
 		return NULL;
 	}
 
@@ -567,8 +566,8 @@ runlist_element *ntfs_runlists_merge(runlist_element *drl,
 	/* Sanity check for illegal overlaps. */
 	if ((drl[di].vcn == srl[si].vcn) && (drl[di].lcn >= 0) &&
 			(srl[si].lcn >= 0)) {
-		ntfs_log_debug("Run lists overlap. Cannot merge!\n");
 		errno = ERANGE;
+		ntfs_log_perror("Run lists overlap. Cannot merge");
 		return NULL;
 	}
 
@@ -1642,16 +1641,16 @@ int ntfs_rl_sparse(runlist *rl)
 	runlist *rlc;
 
 	if (!rl) {
-		ntfs_log_trace("Invalid argument passed.\n");
 		errno = EINVAL;
+		ntfs_log_perror("%s: ", __FUNCTION__);
 		return -1;
 	}
 
 	for (rlc = rl; rlc->length; rlc++)
 		if (rlc->lcn < 0) {
 			if (rlc->lcn != LCN_HOLE) {
-				ntfs_log_trace("Received unmapped runlist.\n");
 				errno = EINVAL;
+				ntfs_log_perror("%s: bad runlist", __FUNCTION__);
 				return -1;
 			}
 			return 1;
@@ -1672,16 +1671,16 @@ s64 ntfs_rl_get_compressed_size(ntfs_volume *vol, runlist *rl)
 	s64 ret = 0;
 
 	if (!rl) {
-		ntfs_log_trace("Invalid argument passed.\n");
 		errno = EINVAL;
+		ntfs_log_perror("%s: ", __FUNCTION__);
 		return -1;
 	}
 
 	for (rlc = rl; rlc->length; rlc++) {
 		if (rlc->lcn < 0) {
 			if (rlc->lcn != LCN_HOLE) {
-				ntfs_log_trace("Received unmapped runlist.\n");
 				errno = EINVAL;
+				ntfs_log_perror("%s: bad runlist", __FUNCTION__);
 				return -1;
 			}
 		} else
