@@ -92,6 +92,12 @@ typedef enum {
 	FSTYPE_FUSEBLK
 } fuse_fstype;
 
+typedef enum {
+	ATIME_ENABLED,
+	ATIME_DISABLED,
+	ATIME_RELATIVE
+} ntfs_atime_t;
+
 typedef struct {
 	fuse_fill_dir_t filler;
 	void *buf;
@@ -105,27 +111,23 @@ typedef enum {
 
 typedef struct {
 	ntfs_volume *vol;
-	int state;
-	long free_mft;
 	unsigned int uid;
 	unsigned int gid;
 	unsigned int fmask;
 	unsigned int dmask;
 	ntfs_fuse_streams_interface streams;
+	ntfs_atime_t atime;
 	BOOL ro;
 	BOOL show_sys_files;
 	BOOL silent;
 	BOOL force;
 	BOOL debug;
-	BOOL noatime;
 	BOOL no_detach;
+	BOOL blkdev;
+	BOOL mounted;
+	struct fuse_chan *fc;
 } ntfs_fuse_context_t;
 
-typedef enum {
-	NF_FreeMFTOutdate	= (1 << 1),  /* Information about amount of
-						free MFT records is outdated. */
-} ntfs_fuse_state_bits;
-
 static struct options {
 	char	*mnt_point;	/* Mount point */
 	char	*options;	/* Mount options */
@@ -142,10 +144,6 @@ static const char *locale_msg =
 "         be correct or visible. Please see the potential solution at\n"
 "         http://ntfs-3g.org/support.html#locale\n";
 
-static const char *dev_fuse_msg =
-"HINT: You should be root, or make ntfs-3g setuid root, or load the FUSE\n"
-"      kernel module as root (modprobe fuse) and make sure /dev/fuse exist.\n";
-
 static const char *usage_msg = 
 "\n"
 "%s %s - Third Generation NTFS Driver\n"
@@ -162,11 +160,6 @@ static const char *usage_msg =
 "\n"
 "%s";
 
-static __inline__ void ntfs_fuse_mark_free_space_outdated(void)
-{
-	/* Mark information about free MFT records outdated. */
-	ctx->state |= NF_FreeMFTOutdate;
-}
 
 /**
  * ntfs_fuse_is_named_data_stream - check path to be to named data stream
@@ -174,75 +167,31 @@ static __inline__ void ntfs_fuse_mark_free_space_outdated(void)
  *
  * Returns 1 if path is to named data stream or 0 otherwise.
  */
-static __inline__ int ntfs_fuse_is_named_data_stream(const char *path)
+static int ntfs_fuse_is_named_data_stream(const char *path)
 {
 	if (strchr(path, ':') && ctx->streams == NF_STREAMS_INTERFACE_WINDOWS)
 		return 1;
 	return 0;
 }
 
-static long ntfs_fuse_get_nr_free_mft_records(ntfs_volume *vol, s64 numof_inode)
+static void ntfs_fuse_update_times(ntfs_inode *ni, ntfs_time_update_flags mask)
 {
-	u8 *buf;
-	long nr_free = 0;
-	s64 br, total = 0;
-
-	if (!(ctx->state & NF_FreeMFTOutdate))
-		return ctx->free_mft;
-	buf = ntfs_malloc(vol->cluster_size);
-	if (!buf)
-		return -errno;
-	while (1) {
-		int i, j;
-
-		br = ntfs_attr_pread(vol->mftbmp_na, total,
-				     vol->cluster_size, buf);
-		if (br <= 0)
-			break;
-		total += br;
-		for (i = 0; i < br; i++)
-			for (j = 0; j < 8; j++) {
-				
-				if (--numof_inode < 0)
-					break;
-				
-				if (!((buf[i] >> j) & 1))
-					nr_free++;
-			}
-	}
-	free(buf);
-	if (!total || br < 0)
-		return -errno;
-	ctx->free_mft = nr_free;
-	ctx->state &= ~(NF_FreeMFTOutdate);
-	return nr_free;
+	if (ctx->atime == ATIME_DISABLED)
+		mask &= ~NTFS_UPDATE_ATIME;
+	else if (ctx->atime == ATIME_RELATIVE && mask == NTFS_UPDATE_ATIME &&
+			ni->last_access_time >= ni->last_data_change_time &&
+			ni->last_access_time >= ni->last_mft_change_time)
+		return;
+	ntfs_inode_update_times(ni, mask);
 }
 
-static long ntfs_get_nr_free_clusters(ntfs_volume *vol)
+static s64 ntfs_get_nr_free_mft_records(ntfs_volume *vol)
 {
-	u8 *buf;
-	long nr_free = 0;
-	s64 br, total = 0;
+	ntfs_attr *na = vol->mftbmp_na;
+	s64 nr_free = ntfs_attr_get_free_bits(na);
 
-	buf = ntfs_malloc(vol->cluster_size);
-	if (!buf)
-		return -errno;
-	while (1) {
-		int i, j;
-
-		br = ntfs_attr_pread(vol->lcnbmp_na, total,
-				     vol->cluster_size, buf);
-		if (br <= 0)
-			break;
-		total += br;
-		for (i = 0; i < br; i++)
-			for (j = 0; j < 8; j++)
-				if (!((buf[i] >> j) & 1))
-					nr_free++;
-	}
-	free(buf);
-	if (!total || br < 0)
-		return -errno;
+	if (nr_free >= 0)
+		nr_free += (na->allocated_size - na->data_size) << 3;
 	return nr_free;
 }
 
@@ -264,32 +213,33 @@ static long ntfs_get_nr_free_clusters(ntfs_volume *vol)
  * Returns 0 on success or -errno on error.
  */
 static int ntfs_fuse_statfs(const char *path __attribute__((unused)),
-		struct statvfs *sfs)
+			    struct statvfs *sfs)
 {
-	long size, delta_bits;
-	u64 allocated_inodes;
+	s64 size;
+	int delta_bits;
 	ntfs_volume *vol;
 
 	vol = ctx->vol;
 	if (!vol)
 		return -ENODEV;
 	
-	/* Optimal transfer block size. */
+	/* File system block size, used for optimal transfer block size. */
 	sfs->f_bsize = vol->cluster_size;
+	
+	/* Fundamental file system block size, used as the unit. */
 	sfs->f_frsize = vol->cluster_size;
+	
 	/*
-	 * Total data blocks in file system in units of f_bsize and since
-	 * inodes are also stored in data blocs ($MFT is a file) this is just
-	 * the total clusters.
+	 * Total number of blocks on file system in units of f_frsize.
+	 * Since inodes are also stored in blocks ($MFT is a file) hence
+	 * this is the number of clusters on the volume.
 	 */
 	sfs->f_blocks = vol->nr_clusters;
 	
-	/* Free data blocks in file system in units of f_bsize. */
+	/* Free blocks available for all and for non-privileged processes. */
 	size = vol->free_clusters;
 	if (size < 0)
 		size = 0;
-	
-	/* Free blocks avail to non-superuser, same as above on NTFS. */
 	sfs->f_bavail = sfs->f_bfree = size;
 	
 	/* Free inodes on the free space */
@@ -299,16 +249,14 @@ static int ntfs_fuse_statfs(const char *path __attribute__((unused)),
 	else
 		size >>= -delta_bits;
 	
-	/* Number of inodes in file system (at this point in time). */
-	allocated_inodes = vol->mft_na->data_size >> vol->mft_record_size_bits;
-	sfs->f_files = allocated_inodes + size; 
+	/* Number of inodes at this point in time. */
+	sfs->f_files = (vol->mftbmp_na->allocated_size << 3) + size;
 	
-	/* Free inodes in fs (based on current total count). */
-	size = ntfs_fuse_get_nr_free_mft_records(vol, allocated_inodes) + size;
+	/* Free inodes available for all and for non-privileged processes. */
+	size += vol->free_mft_records;
 	if (size < 0)
 		size = 0;
-	sfs->f_ffree = size;
-	sfs->f_favail = 0;
+	sfs->f_ffree = sfs->f_favail = size;
 	
 	/* Maximum length of filenames. */
 	sfs->f_namemax = NTFS_MAX_NAME_LEN;
@@ -477,7 +425,7 @@ static int ntfs_fuse_getattr(const char *org_path, struct stat *stbuf)
 	stbuf->st_ctime = ni->last_mft_change_time;
 	stbuf->st_mtime = ni->last_data_change_time;
 exit:
-	if (ni && ntfs_inode_close(ni))
+	if (ntfs_inode_close(ni))
 		set_fuse_error(&res);
 	free(path);
 	if (stream_name_len)
@@ -553,7 +501,7 @@ exit:
 		free(intx_file);
 	if (na)
 		ntfs_attr_close(na);
-	if (ni && ntfs_inode_close(ni))
+	if (ntfs_inode_close(ni))
 		set_fuse_error(&res);
 	free(path);
 	if (stream_name_len)
@@ -589,6 +537,11 @@ static int ntfs_fuse_filler(ntfs_fuse_fill_context_t *fill_ctx,
 	if (MREF(mref) == FILE_root || MREF(mref) >= FILE_first_user ||
 			ctx->show_sys_files) {
 		struct stat st = { .st_ino = MREF(mref) };
+		 
+		if (dt_type == NTFS_DT_REG)
+			st.st_mode = S_IFREG | (0777 & ~ctx->fmask);
+		else if (dt_type == NTFS_DT_DIR)
+			st.st_mode = S_IFDIR | (0777 & ~ctx->dmask); 
 		
 		ret = fill_ctx->filler(fill_ctx->buf, filename, &st, 0);
 	}
@@ -616,6 +569,7 @@ static int ntfs_fuse_readdir(const char *path, void *buf,
 	if (ntfs_readdir(ni, &pos, &fill_ctx,
 			(ntfs_filldir_t)ntfs_fuse_filler))
 		err = -errno;
+	ntfs_fuse_update_times(ni, NTFS_UPDATE_ATIME);
 	if (ntfs_inode_close(ni))
 		set_fuse_error(&err);
 	return err;
@@ -658,18 +612,20 @@ static int ntfs_fuse_open(const char *org_path,
 static int ntfs_fuse_read(const char *org_path, char *buf, size_t size,
 		off_t offset, struct fuse_file_info *fi __attribute__((unused)))
 {
-	ntfs_volume *vol;
 	ntfs_inode *ni = NULL;
 	ntfs_attr *na = NULL;
 	char *path = NULL;
 	ntfschar *stream_name;
-	int stream_name_len, res, total = 0;
+	int stream_name_len, res;
+	s64 total = 0;
+
+	if (!size)
+		return 0;
 
 	stream_name_len = ntfs_fuse_parse_path(org_path, &path, &stream_name);
 	if (stream_name_len < 0)
 		return stream_name_len;
-	vol = ctx->vol;
-	ni = ntfs_pathname_to_inode(vol, NULL, path);
+	ni = ntfs_pathname_to_inode(ctx->vol, NULL, path);
 	if (!ni) {
 		res = -errno;
 		goto exit;
@@ -679,26 +635,32 @@ static int ntfs_fuse_read(const char *org_path, char *buf, size_t size,
 		res = -errno;
 		goto exit;
 	}
-	if (offset + size > na->data_size)
+	if (offset + size > na->data_size) {
+		if (na->data_size < offset)
+			goto ok;
 		size = na->data_size - offset;
-	while (size) {
-		res = ntfs_attr_pread(na, offset, size, buf);
-		if (res < (s64)size)
-			ntfs_log_perror("ntfs_attr_pread partial write (%lld: "
-				"%lld <> %d)", (s64)offset, (s64)size, res);
-		if (res <= 0) {
-			res = -errno;
+	}
+	while (size > 0) {
+		s64 ret = ntfs_attr_pread(na, offset, size, buf);
+		if (ret != (s64)size)
+			ntfs_log_perror("ntfs_attr_pread error reading '%s' at "
+				"offset %lld: %lld <> %lld", org_path, 
+				(long long)offset, (long long)size, (long long)ret);
+		if (ret <= 0 || ret > (s64)size) {
+			res = (ret < 0) ? -errno : -EIO;
 			goto exit;
 		}
-		size -= res;
-		offset += res;
-		total += res;
+		size -= ret;
+		offset += ret;
+		total += ret;
 	}
+ok:
+	ntfs_fuse_update_times(na->ni, NTFS_UPDATE_ATIME);
 	res = total;
 exit:
 	if (na)
 		ntfs_attr_close(na);
-	if (ni && ntfs_inode_close(ni))
+	if (ntfs_inode_close(ni))
 		set_fuse_error(&res);
 	free(path);
 	if (stream_name_len)
@@ -746,11 +708,12 @@ static int ntfs_fuse_write(const char *org_path, const char *buf, size_t size,
 		total += res;
 	}
 	res = total;
+	if (res > 0)
+		ntfs_fuse_update_times(na->ni, NTFS_UPDATE_MCTIME);
 exit:
-	ntfs_fuse_mark_free_space_outdated();
 	if (na)
 		ntfs_attr_close(na);
-	if (ni && ntfs_inode_close(ni))
+	if (ntfs_inode_close(ni))
 		set_fuse_error(&res);
 	free(path);
 	if (stream_name_len)
@@ -763,7 +726,7 @@ static int ntfs_fuse_truncate(const char *org_path, off_t size)
 {
 	ntfs_volume *vol;
 	ntfs_inode *ni = NULL;
-	ntfs_attr *na;
+	ntfs_attr *na = NULL;
 	int res;
 	char *path = NULL;
 	ntfschar *stream_name;
@@ -784,12 +747,12 @@ static int ntfs_fuse_truncate(const char *org_path, off_t size)
 	if (ntfs_attr_truncate(na, size))
 		goto exit;
 	
-	ntfs_fuse_mark_free_space_outdated();
-	ntfs_attr_close(na);
+	ntfs_fuse_update_times(na->ni, NTFS_UPDATE_MCTIME);
 	errno = 0;
 exit:
 	res = -errno;
-	if (ni && ntfs_inode_close(ni))
+	ntfs_attr_close(na);
+	if (ntfs_inode_close(ni))
 		set_fuse_error(&res);
 	free(path);
 	if (stream_name_len)
@@ -868,11 +831,12 @@ static int ntfs_fuse_create(const char *org_path, dev_t type, dev_t dev,
 	if (ni) {
 		if (ntfs_inode_close(ni))
 			set_fuse_error(&res);
+		ntfs_fuse_update_times(dir_ni, NTFS_UPDATE_MCTIME);
 	} else
 		res = -errno;
 exit:
 	free(uname);
-	if (dir_ni && ntfs_inode_close(dir_ni))
+	if (ntfs_inode_close(dir_ni))
 		set_fuse_error(&res);
 	if (utarget)
 		free(utarget);
@@ -910,6 +874,28 @@ static int ntfs_fuse_create_stream(const char *path,
 	return res;
 }
 
+static int ntfs_fuse_create_file(const char *org_path, mode_t mode, 
+			struct fuse_file_info *fi __attribute__((unused)))
+{
+	char *path = NULL;
+	ntfschar *stream_name;
+	int stream_name_len;
+	int res;
+
+	stream_name_len = ntfs_fuse_parse_path(org_path, &path, &stream_name);
+	if (stream_name_len < 0)
+		return stream_name_len;
+	if (!stream_name_len)
+		res = ntfs_fuse_create(path, mode & S_IFMT, 0, NULL);
+	else
+		res = ntfs_fuse_create_stream(path, stream_name,
+				stream_name_len);
+	free(path);
+	if (stream_name_len)
+		free(stream_name);
+	return res;
+}
+
 static int ntfs_fuse_mknod(const char *org_path, mode_t mode, dev_t dev)
 {
 	char *path = NULL;
@@ -929,7 +915,6 @@ static int ntfs_fuse_mknod(const char *org_path, mode_t mode, dev_t dev)
 	else
 		res = ntfs_fuse_create_stream(path, stream_name,
 				stream_name_len);
-	ntfs_fuse_mark_free_space_outdated();
 exit:
 	free(path);
 	if (stream_name_len)
@@ -941,17 +926,10 @@ static int ntfs_fuse_symlink(const char *to, const char *from)
 {
 	if (ntfs_fuse_is_named_data_stream(from))
 		return -EINVAL; /* n/a for named data streams. */
-	ntfs_fuse_mark_free_space_outdated();
 	return ntfs_fuse_create(from, S_IFLNK, 0, to);
 }
 
-/**
- * NOTE: About the role of mtime: during rename(3), which is currently 
- * implemented by the help of link() operations, modification time mustn't
- * be updated, so we NInoSetNoMtimeUpdate() such inodes after they are opened.
- * This is not very nice itself but it may be eliminated, in time.
- */
-static int ntfs_fuse_ln(const char *old_path, const char *new_path, int mtime)
+static int ntfs_fuse_link(const char *old_path, const char *new_path)
 {
 	char *name;
 	ntfschar *uname = NULL;
@@ -973,9 +951,6 @@ static int ntfs_fuse_ln(const char *old_path, const char *new_path, int mtime)
 		goto exit;
 	}
 	
-	if (!mtime)
-		NInoSetNoMtimeUpdate(ni);
-	
 	/* Generate unicode filename. */
 	name = strrchr(path, '/');
 	name++;
@@ -991,29 +966,28 @@ static int ntfs_fuse_ln(const char *old_path, const char *new_path, int mtime)
 		res = -errno;
 		goto exit;
 	}
-	ntfs_fuse_mark_free_space_outdated();
-	/* Create hard link. */
-	if (ntfs_link(ni, dir_ni, uname, uname_len))
+
+	if (ntfs_link(ni, dir_ni, uname, uname_len)) {
 		res = -errno;
+		goto exit;
+	}
+
+	ntfs_fuse_update_times(ni, NTFS_UPDATE_CTIME);
+	ntfs_fuse_update_times(dir_ni, NTFS_UPDATE_MCTIME);
 exit:
 	/* 
 	 * Must close dir_ni first otherwise ntfs_inode_sync_file_name(ni)
 	 * may fail because ni may not be in parent's index on the disk yet.
 	 */
-	if (dir_ni && ntfs_inode_close(dir_ni))
+	if (ntfs_inode_close(dir_ni))
 		set_fuse_error(&res);
-	if (ni && ntfs_inode_close(ni))
+	if (ntfs_inode_close(ni))
 		set_fuse_error(&res);
 	free(uname);
 	free(path);
 	return res;
 }
 
-static int ntfs_fuse_link(const char *old_path, const char *new_path)
-{
-	return ntfs_fuse_ln(old_path, new_path, 1);
-}
-
 static int ntfs_fuse_rm(const char *org_path)
 {
 	char *name;
@@ -1046,16 +1020,17 @@ static int ntfs_fuse_rm(const char *org_path)
 		res = -errno;
 		goto exit;
 	}
-	/* Delete object. */
+	
 	if (ntfs_delete(ni, dir_ni, uname, uname_len))
 		res = -errno;
-	ni = NULL;
+	/* ntfs_delete() always closes ni and dir_ni */
+	ni = dir_ni = NULL;
 exit:
-	if (ni && ntfs_inode_close(ni))
+	if (ntfs_inode_close(dir_ni))
+		set_fuse_error(&res);
+	if (ntfs_inode_close(ni))
 		set_fuse_error(&res);
 	free(uname);
-	if (dir_ni && ntfs_inode_close(dir_ni))
-		set_fuse_error(&res);
 	free(path);
 	return res;
 }
@@ -1092,7 +1067,6 @@ static int ntfs_fuse_unlink(const char *org_path)
 		res = ntfs_fuse_rm(path);
 	else
 		res = ntfs_fuse_rm_stream(path, stream_name, stream_name_len);
-	ntfs_fuse_mark_free_space_outdated();
 	free(path);
 	if (stream_name_len)
 		free(stream_name);
@@ -1107,14 +1081,14 @@ static int ntfs_fuse_safe_rename(const char *old_path,
 
 	ntfs_log_trace("Entering\n");
 	
-	ret = ntfs_fuse_ln(new_path, tmp, 0);
+	ret = ntfs_fuse_link(new_path, tmp);
 	if (ret)
 		return ret;
 	
 	ret = ntfs_fuse_unlink(new_path);
 	if (!ret) {
 		
-		ret = ntfs_fuse_ln(old_path, new_path, 0);
+		ret = ntfs_fuse_link(old_path, new_path);
 		if (ret)
 			goto restore;
 		
@@ -1128,7 +1102,7 @@ static int ntfs_fuse_safe_rename(const char *old_path,
 	
 	goto cleanup;
 restore:
-	if (ntfs_fuse_ln(tmp, new_path, 0)) {
+	if (ntfs_fuse_link(tmp, new_path)) {
 err:
 		ntfs_log_perror("Rename failed. Existing file '%s' was renamed "
 				"to '%s'", new_path, tmp);
@@ -1197,7 +1171,7 @@ static int ntfs_fuse_rename(const char *old_path, const char *new_path)
 		goto out;
 	}
 
-	ret = ntfs_fuse_ln(old_path, new_path, 0);
+	ret = ntfs_fuse_link(old_path, new_path);
 	if (ret)
 		goto out;
 	
@@ -1216,7 +1190,6 @@ static int ntfs_fuse_mkdir(const char *path,
 {
 	if (ntfs_fuse_is_named_data_stream(path))
 		return -EINVAL; /* n/a for named data streams. */
-	ntfs_fuse_mark_free_space_outdated();
 	return ntfs_fuse_create(path, S_IFDIR, 0, NULL);
 }
 
@@ -1224,7 +1197,6 @@ static int ntfs_fuse_rmdir(const char *path)
 {
 	if (ntfs_fuse_is_named_data_stream(path))
 		return -EINVAL; /* n/a for named data streams. */
-	ntfs_fuse_mark_free_space_outdated();
 	return ntfs_fuse_rm(path);
 }
 
@@ -1239,22 +1211,13 @@ static int ntfs_fuse_utime(const char *path, struct utimbuf *buf)
 	if (!ni)
 		return -errno;
 	
-	NInoSetNoParentMtimeUpdate(ni);
-	
 	if (buf) {
 		ni->last_access_time = buf->actime;
 		ni->last_data_change_time = buf->modtime;
-		ni->last_mft_change_time = buf->modtime;
-	} else {
-		time_t now;
+		ntfs_fuse_update_times(ni, NTFS_UPDATE_CTIME);
+	} else
+		ntfs_inode_update_times(ni, NTFS_UPDATE_AMCTIME);
 
-		now = time(NULL);
-		ni->last_access_time = now;
-		ni->last_data_change_time = now;
-		ni->last_mft_change_time = now;
-	}
-	NInoFileNameSetDirty(ni);
-	NInoSetDirty(ni);
 	if (ntfs_inode_close(ni))
 		set_fuse_error(&res);
 	return res;
@@ -1417,7 +1380,7 @@ static int ntfs_fuse_getxattr_windows(const char *path, const char *name,
 			if ((size_t)ret <= size) {
 				/* Don't add space to the beginning of line. */
 				if (to != value) {
-					*to = ' ';
+					*to = '\0';
 					to++;
 				}
 				strncpy(to, tmp_name, tmp_name_len);
@@ -1520,7 +1483,6 @@ static int ntfs_fuse_setxattr(const char *path, const char *name,
 		res = -EEXIST;
 		goto exit;
 	}
-	ntfs_fuse_mark_free_space_outdated();
 	if (!na) {
 		if (flags == XATTR_REPLACE) {
 			res = -ENODATA;
@@ -1580,7 +1542,6 @@ static int ntfs_fuse_removexattr(const char *path, const char *name)
 		res = -errno;
 	}
 	
-	ntfs_fuse_mark_free_space_outdated();
 exit:
 	free(lename);
 	if (ntfs_inode_close(ni))
@@ -1590,29 +1551,30 @@ exit:
 
 #endif /* HAVE_SETXATTR */
 
-static void ntfs_fuse_destroy(void)
+static void ntfs_close(void)
 {
 	if (!ctx)
 		return;
 	
-	if (ctx->vol) {
-		ntfs_log_info("Unmounting %s (%s)\n", opts.device,
+	if (!ctx->vol)
+		return;
+	
+	if (ctx->mounted)
+		ntfs_log_info("Unmounting %s (%s)\n", opts.device, 
 			      ctx->vol->vol_name);
-		if (ntfs_umount(ctx->vol, FALSE))
-			ntfs_log_perror("Failed to cleanly unmount volume %s",
-					opts.device);
-	}
-	free(ctx);
-	ctx = NULL;
-	free(opts.device);
+	
+	if (ntfs_umount(ctx->vol, FALSE))
+		ntfs_log_perror("Failed to close volume %s", opts.device);
+	
+	ctx->vol = NULL;
 }
 
 static void ntfs_fuse_destroy2(void *unused __attribute__((unused)))
 {
-	ntfs_fuse_destroy();
+	ntfs_close();
 }
 
-static struct fuse_operations ntfs_fuse_oper = {
+static struct fuse_operations ntfs_3g_ops = {
 	.getattr	= ntfs_fuse_getattr,
 	.readlink	= ntfs_fuse_readlink,
 	.readdir	= ntfs_fuse_readdir,
@@ -1623,6 +1585,7 @@ static struct fuse_operations ntfs_fuse_oper = {
 	.statfs		= ntfs_fuse_statfs,
 	.chmod		= ntfs_fuse_chmod,
 	.chown		= ntfs_fuse_chown,
+	.create		= ntfs_fuse_create_file,
 	.mknod		= ntfs_fuse_mknod,
 	.symlink	= ntfs_fuse_symlink,
 	.link		= ntfs_fuse_link,
@@ -1643,36 +1606,46 @@ static struct fuse_operations ntfs_fuse_oper = {
 
 static int ntfs_fuse_init(void)
 {
-	ctx = ntfs_malloc(sizeof(ntfs_fuse_context_t));
+	ctx = ntfs_calloc(sizeof(ntfs_fuse_context_t));
 	if (!ctx)
 		return -1;
 	
 	*ctx = (ntfs_fuse_context_t) {
-		.state = NF_FreeMFTOutdate,
 		.uid = getuid(),
 		.gid = getgid(),
-		.fmask = 0,
-		.dmask = 0,
 		.streams = NF_STREAMS_INTERFACE_NONE,
 	};
 	return 0;
 }
 
-static ntfs_volume *ntfs_open(const char *device, char *mntpoint, int blkdev)
+static int ntfs_open(const char *device, char *mntpoint)
 {
 	unsigned long flags = 0;
 	
-	if (!blkdev)
+	if (!ctx->blkdev)
 		flags |= MS_EXCLUSIVE;
 	if (ctx->ro)
 		flags |= MS_RDONLY;
-	if (ctx->noatime)
-		flags |= MS_NOATIME;
 	if (ctx->force)
 		flags |= MS_FORCE;
 
-	ctx->vol = utils_mount_volume(device, mntpoint, flags, ctx->force);
-	return ctx->vol;
+	ctx->vol = utils_mount_volume(device, mntpoint, flags);
+	if (!ctx->vol)
+		return -1;
+	
+	ctx->vol->free_clusters = ntfs_attr_get_free_bits(ctx->vol->lcnbmp_na);
+	if (ctx->vol->free_clusters < 0) {
+		ntfs_log_perror("Failed to read NTFS $Bitmap");
+		return -1;
+	}
+
+	ctx->vol->free_mft_records = ntfs_get_nr_free_mft_records(ctx->vol);
+	if (ctx->vol->free_mft_records < 0) {
+		ntfs_log_perror("Failed to calculate free MFT records");
+		return -1;
+	}
+	
+	return 0;
 }
 
 static void signal_handler(int arg __attribute__((unused)))
@@ -1707,15 +1680,10 @@ static char *parse_mount_options(const char *orig_opts)
 		return NULL;
 	}
 	
-	/*
-	 * FIXME: Due to major performance hit and interference
-	 * issues, always use the 'noatime' options for now.
-	 */
-	ctx->noatime = TRUE;
-	strcat(ret, "noatime,");
-	
 	ctx->silent = TRUE;
 	
+	ctx->atime = ATIME_RELATIVE;
+	
 	s = options;
 	while (s && *s && (val = strsep(&s, ","))) {
 		opt = strsep(&val, "=");
@@ -1733,6 +1701,21 @@ static char *parse_mount_options(const char *orig_opts)
 						"have value.\n");
 				goto err_exit;
 			}
+			ctx->atime = ATIME_DISABLED;
+		} else if (!strcmp(opt, "atime")) {
+			if (val) {
+				ntfs_log_error("'atime' option should not "
+						"have value.\n");
+				goto err_exit;
+			}
+			ctx->atime = ATIME_ENABLED;
+		} else if (!strcmp(opt, "relatime")) {
+			if (val) {
+				ntfs_log_error("'relatime' option should not "
+						"have value.\n");
+				goto err_exit;
+			}
+			ctx->atime = ATIME_RELATIVE;
 		} else if (!strcmp(opt, "fake_rw")) {
 			if (val) {
 				ntfs_log_error("'fake_rw' option should not "
@@ -1885,6 +1868,14 @@ static char *parse_mount_options(const char *orig_opts)
 		strcat(ret, def_opts);
 	if (default_permissions)
 		strcat(ret, "default_permissions,");
+	
+	if (ctx->atime == ATIME_RELATIVE)
+		strcat(ret, "relatime,");
+	else if (ctx->atime == ATIME_ENABLED)
+		strcat(ret, "atime,");
+	else
+		strcat(ret, "noatime,");
+	
 	strcat(ret, "fsname=");
 	strcat(ret, opts.device);
 exit:
@@ -2024,7 +2015,13 @@ static int parse_options(int argc, char *argv[])
 	return 0;
 }
 
-#ifdef linux
+#if defined(linux) || defined(__uClinux__)
+
+static const char *dev_fuse_msg =
+"HINT: You should be root, or make ntfs-3g setuid root, or load the FUSE\n"
+"      kernel module as root ('modprobe fuse' or 'insmod <path_to>/fuse.ko'"
+"      or insmod <path_to>/fuse.o'). Make also sure that the fuse device"
+"      exists. It's usually either /dev/fuse or /dev/misc/fuse.";
 
 static const char *fuse26_kmod_msg =
 "WARNING: Deficient Linux kernel detected. Some driver features are\n"
@@ -2037,19 +2034,39 @@ static const char *fuse26_kmod_msg =
 "         http://ntfs-3g.org/support.html#fuse26\n"
 "\n";
 
-static void create_dev_fuse(void)
+static void mknod_dev_fuse(const char *dev)
 {
 	struct stat st;
 	
-	if (stat("/dev/fuse", &st) && (errno == ENOENT)) {
-		if (mknod("/dev/fuse", S_IFCHR | 0666, makedev(10, 229))) {
-			ntfs_log_perror("Failed to create /dev/fuse");
+	if (stat(dev, &st) && (errno == ENOENT)) {
+		mode_t mask = umask(0); 
+		if (mknod(dev, S_IFCHR | 0666, makedev(10, 229))) {
+			ntfs_log_perror("Failed to create '%s'", dev);
 			if (errno == EPERM)
 				ntfs_log_error(dev_fuse_msg);
 		}
+		umask(mask);
 	}
 }
 
+static void create_dev_fuse(void)
+{
+	mknod_dev_fuse("/dev/fuse");
+
+#ifdef __UCLIBC__
+	{
+		struct stat st;
+		/* The fuse device is under /dev/misc using devfs. */
+		if (stat("/dev/misc", &st) && (errno == ENOENT)) {
+			mode_t mask = umask(0); 
+			mkdir("/dev/misc", 0775);
+			umask(mask);
+		}
+		mknod_dev_fuse("/dev/misc/fuse");
+	}
+#endif
+}
+
 static fuse_fstype get_fuse_fstype(void)
 {
 	char buf[256];
@@ -2123,8 +2140,6 @@ static struct fuse_chan *try_fuse_mount(char *parsed_options)
 	}
 	
 	fc = fuse_mount(opts.mnt_point, &margs);
-	if (!fc)
-		ntfs_log_error("FUSE mount point creation failed\n");
 free_args:
 	fuse_opt_free_args(&margs);
 	return fc;
@@ -2170,15 +2185,46 @@ static void set_user_mount_option(char *parsed_options, uid_t uid)
 	strcat(parsed_options, option);
 }
 
+static struct fuse *mount_fuse(char *parsed_options)
+{
+	struct fuse *fh = NULL;
+	struct fuse_args args = FUSE_ARGS_INIT(0, NULL);
+	
+	/* Libfuse can't always find fusermount, so let's help it. */
+	if (setenv("PATH", ":/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin", 0))
+		ntfs_log_perror("WARNING: Failed to set $PATH\n");
+	
+	ctx->fc = try_fuse_mount(parsed_options);
+	if (!ctx->fc)
+		return NULL;
+	
+	if (fuse_opt_add_arg(&args, "") == -1)
+		goto err;
+	if (fuse_opt_add_arg(&args, "-ouse_ino,kernel_cache") == -1)
+		goto err;
+	if (ctx->debug)
+		if (fuse_opt_add_arg(&args, "-odebug") == -1)
+			goto err;
+	
+	fh = fuse_new(ctx->fc, &args , &ntfs_3g_ops, sizeof(ntfs_3g_ops), NULL);
+	if (!fh)
+		goto err;
+	
+	ctx->mounted = TRUE;
+out:
+	fuse_opt_free_args(&args);
+	return fh;
+err:	
+	fuse_unmount(opts.mnt_point, ctx->fc);
+	goto out;
+}
+
 int main(int argc, char *argv[])
 {
 	char *parsed_options = NULL;
-	struct fuse_args margs = FUSE_ARGS_INIT(0, NULL);
 	struct fuse *fh;
-	struct fuse_chan *fc;
 	fuse_fstype fstype = FSTYPE_UNKNOWN;
 	struct stat sbuf;
-	int use_blkdev = 0;
 	uid_t uid, euid;
 	int err = 10;
 
@@ -2207,7 +2253,7 @@ int main(int argc, char *argv[])
 		goto err_out;
 	}
 
-#ifdef linux
+#if defined(linux) || defined(__uClinux__)
 	fstype = get_fuse_fstype();
 	if (fstype == FSTYPE_NONE || fstype == FSTYPE_UNKNOWN)
 		fstype = load_fuse_module();
@@ -2221,58 +2267,27 @@ int main(int argc, char *argv[])
 	}
 	/* Always use fuseblk for block devices unless it's surely missing. */
 	if (S_ISBLK(sbuf.st_mode) && (fstype != FSTYPE_FUSE))
-		use_blkdev = 1;
+		ctx->blkdev = TRUE;
 
-	if (!ntfs_open(opts.device, opts.mnt_point, use_blkdev))
+	if (ntfs_open(opts.device, opts.mnt_point))
 		goto err_out;
 	
-	ctx->vol->free_clusters = ntfs_get_nr_free_clusters(ctx->vol);
-	if (ctx->vol->free_clusters < 0) {
-		ntfs_log_perror("Failed to read NTFS $Bitmap");
+	if (ctx->blkdev) {
+		/* Must do after ntfs_open() to set the right blksize. */
+		set_fuseblk_options(parsed_options);
+		set_user_mount_option(parsed_options, uid);
+	}
+	
+	fh = mount_fuse(parsed_options);
+	if (!fh)
 		goto err_out;
-	}
-	
-	if (use_blkdev) {
-	    set_fuseblk_options(parsed_options);
-	    set_user_mount_option(parsed_options, uid);
-	}
-	
-	/* Libfuse can't always find fusermount, so let's help it. */
-	if (setenv("PATH", ":/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin", 0))
-		ntfs_log_perror("WARNING: Failed to set $PATH\n");
-	
-	fc = try_fuse_mount(parsed_options);
-	if (!fc)
-		goto err_out;
-	
-	fh = (struct fuse *)1; /* Cast anything except NULL to handle errors. */
-	if (fuse_opt_add_arg(&margs, "") == -1 ||
-	    fuse_opt_add_arg(&margs, "-o") == -1)
-		    fh = NULL;
-	if (!ctx->debug && !ctx->no_detach) {
-		if (fuse_opt_add_arg(&margs, "use_ino,kernel_cache") == -1)
-			fh = NULL;
-	} else {
-		if (fuse_opt_add_arg(&margs, "use_ino,kernel_cache,debug") == -1)
-			fh = NULL;
-	}
-	if (fh)
-		fh = fuse_new(fc, &margs , &ntfs_fuse_oper,
-				sizeof(ntfs_fuse_oper), NULL);
-	fuse_opt_free_args(&margs);
-	if (!fh) {
-		ntfs_log_error("fuse_new failed.\n");
-		fuse_unmount(opts.mnt_point, fc);
-		goto err_out;
-	}
-	
+		
 	if (setuid(uid)) {
 		ntfs_log_perror("Failed to set user ID to %d", uid);
-		fuse_unmount(opts.mnt_point, fc);
-		goto err_out;
+		goto err_umount;
 	}
 
-#ifdef linux
+#if defined(linux) || defined(__uClinux__)
 	if (S_ISBLK(sbuf.st_mode) && (fstype == FSTYPE_FUSE))
 		ntfs_log_info(fuse26_kmod_msg);
 #endif	
@@ -2298,12 +2313,15 @@ int main(int argc, char *argv[])
 	
 	fuse_loop(fh);
 	
-	fuse_unmount(opts.mnt_point, fc);
-	fuse_destroy(fh);
 	err = 0;
+err_umount:
+	fuse_unmount(opts.mnt_point, ctx->fc);
+	fuse_destroy(fh);
 err_out:
-	free(opts.options);
+	ntfs_close();
+	free(ctx);
 	free(parsed_options);
-	ntfs_fuse_destroy();
+	free(opts.options);
+	free(opts.device);
 	return err;
 }
